/*
 * Copyright (C) 2001, 2002 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License,
 * The Sun (tm) Java Advanced Imaging library (JAI), The Sun JIMI library
 * (or with modified versions of the above that use the same license as the above),
 * and distribute linked combinations including the two.  You must obey the
 * GNU General Public License in all respects for all of the code used other than
 * the above mentioned libraries.  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your version.
 */
package  mircoders.media;

import mir.config.MirPropertiesConfiguration;
import mir.entity.Entity;
import mir.log.LoggerWrapper;
import mir.media.MediaExc;
import mir.media.MediaFailure;
import mir.misc.StringUtil;
import mir.session.UploadedFile;
import mir.util.IORoutines;

import javax.servlet.ServletContext;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;


/**
 * This is the Generic MediaHandler. It stores the media data on
 * the filesystem and keeps basic metadata  (size, type...) in the
 * DB. Usually only representation needs to be overridden.
 * See the MediaHandlerAudio class to see an example of how one
 * could override it.
 * <p>
 * Most media handlers should override this class.
 * <p>
 * In theory, it could be used to handle miscellaneous media that
 * we don't have entered in the media_type table, (like RTF documents,
 * PS, PDF, etc..)
 * <p>
 * Of course it implements the MirMediaHandler interface.
 *
 * @see mir.media.MediaHandler
 * @author mh <mh@nadir.org>
 * @version $Id: MediaHandlerGeneric.java,v 1.20.2.9 2005/08/21 17:09:23 zapata Exp $
 */

public class MediaHandlerGeneric extends AbstractMediaHandler {
  protected static MirPropertiesConfiguration configuration = MirPropertiesConfiguration.instance();

  protected LoggerWrapper logger = new LoggerWrapper("Media.Generic");

  /** {@inheritDoc} */
  public void store(UploadedFile anUploadedFile, Entity aMedia, Entity aMediaType) throws MediaExc, MediaFailure {
    try {
      anUploadedFile.writeToFile(getMediaStorageFile(aMedia, aMediaType));
      aMedia.setFieldValue("publish_path", getRelativeMediaStoragePath(aMedia, aMediaType));
      aMedia.setFieldValue("size", Long.toString(getMediaStorageFile(aMedia, aMediaType).length()));
      aMedia.update();
    }
    catch (Throwable e) {
      logger.error("MediaHandlerGeneric.set: " + e.toString());
      throw new MediaFailure(e);
    }
  }

  /** {@inheritDoc} */
  public void store(InputStream anInputStream, Entity aMedia, Entity aMediaType) throws MediaExc, MediaFailure {
    try {
      IORoutines.copyStream(anInputStream, new FileOutputStream(getMediaStorageFile(aMedia, aMediaType)));

      aMedia.setFieldValue("publish_path", getRelativeMediaStoragePath(aMedia, aMediaType));
      aMedia.setFieldValue("size", Long.toString(getMediaStorageFile(aMedia, aMediaType).length()));
      aMedia.update();
    }
    catch (Throwable e) {
      logger.error("MediaHandlerGeneric.set: " + e.toString());
      throw new MediaFailure(e);
    }
  }

  /** {@inheritDoc} */
  public void produce(Entity aMedia, Entity aMediaType) throws MediaExc, MediaFailure {
    if (!getMediaStorageFile(aMedia, aMediaType).exists()) {
      throw new MediaExc("error in producing media:: " + getMediaStorageFile(aMedia, aMediaType) + " does not exist!");
    }
  }

  /** {@inheritDoc} */
  public InputStream getMedia(Entity aMedia, Entity aMediaType) throws MediaExc, MediaFailure {
    try {
      return new FileInputStream(getMediaStorageFile(aMedia, aMediaType));
    }
    catch (Throwable e) {
      throw new MediaFailure("MediaHandlerGeneric.getMedia(): " + e.toString(), e);
    }
  }

  public InputStream getThumbnail(Entity ent) throws MediaExc, MediaFailure {
    return null;
  }

  public String getThumbnailMimeType(Entity aMediaEntity, Entity aMediaType) throws MediaExc, MediaFailure {
    ServletContext servletContext = MirPropertiesConfiguration.getContext();
    String fileName = aMediaEntity.getId() + "." + aMediaType.getFieldValue("name");

    return servletContext.getMimeType(fileName);
  }

  public String getBaseStoragePath() {
    return configuration.getString("Producer.Media.Path");
  }

  public String getBaseIconStoragePath() {
    return configuration.getString("Producer.Image.IconPath");
  }

  public String getPublishHost() {
    return StringUtil.removeSlash(configuration.getString("Producer.Media.Host"));
  }

  public String getTinyIconName() {
    return configuration.getString("Producer.Icon.TinyText");
  }

  public String getBigIconName() {
    return configuration.getString("Producer.Icon.BigText");
  }

  public String getIconAltName() {
    return "Generic media";
  }

  public String getDescr(Entity mediaType) {
    return mediaType.getFieldValue("mime_type");
  }

}



